/*******************************************************************************
* Copyright (C) 2005, 2006 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
* Author: Jon Carey (jcarey@novell.com)
******************************************************************************/
#include <cmpiutil/base.h>
#include <cmpiutil/string.h>
#include <cmpiutil/cmpiUtils.h>
#include <cmpiutil/cmpiSimpleAssoc.h>

#include <cmpidt.h>
#include <cmpift.h>
#include <cmpimacs.h>

#include <stdio.h>

#define LOCAL_DEBUG

#ifdef LOCAL_DEBUG
#define DEBUGOUT(fmt, args...) printf(fmt,## args)
#else
#define DEBUGOUT(fmt, args...)
#endif

void
cmpiutilSimpleAssocResultsForAssociators(
		cmpiutilSimpleAssocCtx handler,
		CMPIInstance *inst,
		CMPIStatus *pStatus )
{
	DEBUGOUT("In cmpiutilSimpleAssocResultsForAssociators() called\n");
	// I got an instance of the association.
	// Return an instance of one side of the association
	// determine which side to send back, based on what is in handler->origCop

	// go through association looking for 'ref' keys, then compare
	CMPIObjectPath *incop = CMGetObjectPath(inst, pStatus);
	DEBUGOUT("Namespace of incoming instance:  %s\n",
			CMGetCharPtr(CMGetNameSpace(incop, NULL)));
	unsigned int keycount = CMGetKeyCount(incop, pStatus);

	DEBUGOUT("incoming cop has %d keys\n", keycount);
	DEBUGOUT("handler role: %s\n", handler->role);

	CMPIString *keyname;
	CMPIData key;
	unsigned int i = 0;
	for (i=0; i<keycount; i++)
	{
		DEBUGOUT("looping:  key %d\n", i);
		key = CMGetKeyAt(incop, i, &keyname, pStatus);
		DEBUGOUT("  key name: %s\n", CMGetCharPtr(keyname));
		DEBUGOUT("  key type: %d\n", key.type);
		if (key.type == CMPI_ref)
		{
			DEBUGOUT("  key type == REF... comparing object paths\n");
			DEBUGOUT("  orig cop: %s\n",
					CMGetCharPtr(CMObjectPathToString(handler->origCop, NULL)));
			DEBUGOUT("  key cop:  %s\n",
					CMGetCharPtr(CMObjectPathToString(key.value.ref, NULL)));
			if (!cmpiutilAreObjectPathsSame( handler->origCop,
											key.value.ref,
											pStatus ))
			{
				DEBUGOUT("  object paths not same, returning instance\n");
				// this is my guy... assuming only two refs in the objectPath
				// gotta get the instance
				CMPIInstance *inst = CBGetInstance(handler->broker,
												   handler->ctx,
												   key.value.ref,
												   handler->properties,
											       pStatus);
				CMReturnInstance(handler->results, inst);
				break;
			}
            else
            {
                DEBUGOUT("  objectPaths same, not processing this one\n");
            }
		}
	}
}

void
cmpiutilSimpleAssocResultsForAssociatorNames(
		cmpiutilSimpleAssocCtx handler,
		CMPIInstance *inst,
		CMPIStatus *pStatus )
{
	// I got an instance of the association.
	// Return an objectPath of one side of the association
	// determine which side to send back, based on what is in handler->origCop

	// go through association looking for 'ref' keys, then compare
	CMPIObjectPath *incop = CMGetObjectPath(inst, pStatus);
	unsigned int keycount = CMGetKeyCount(incop, pStatus);

	CMPIString *keyname;
	CMPIData key;
	unsigned int i = 0;
	for (i=0; i<keycount; i++)
	{
		key = CMGetKeyAt(incop, i, &keyname, pStatus);
		if (key.type == CMPI_ref)
		{
			if (!cmpiutilAreObjectPathsSame( handler->origCop,
											key.value.ref,
											pStatus ))
			{
				// this is my guy... assuming only two refs in the objectPath
				// return the object path
				CMReturnObjectPath(handler->results, key.value.ref);
				break;
			}
		}
	}
}

void
cmpiutilSimpleAssocResultsForReferences(
		cmpiutilSimpleAssocCtx handler,
		CMPIInstance *inst,
		CMPIStatus *pStatus )
{
	// I got an instance of the association.
	// Return the same instance
	CMReturnInstance(handler->results, inst);
}

void
cmpiutilSimpleAssocResultsForReferenceNames(
		cmpiutilSimpleAssocCtx handler,
		CMPIInstance *inst,
		CMPIStatus *pStatus )
{
	// I got an instance of the association.
	// Return an object path of this instance
	CMPIObjectPath* op = CMGetObjectPath(inst, pStatus);
	CMReturnObjectPath(handler->results, op);
}

void
cmpiutilSimpleAssocResults(
		cmpiutilSimpleAssocCtx handler,
		CMPIInstance *inst,
		CMPIStatus *pStatus )
{
	DEBUGOUT("In cmpiutilSimpleAssocResults() called\n");
	if (handler != NULL)
	{
		switch (handler->type)
		{
			case ASSOCS:
				cmpiutilSimpleAssocResultsForAssociators(handler,
														inst,
														pStatus);
				break;
			case ASSOCNAMES:
				cmpiutilSimpleAssocResultsForAssociatorNames(handler,
														inst,
														pStatus);
				break;
			case REFS:
				cmpiutilSimpleAssocResultsForReferences(handler, inst, pStatus);
				break;
			case REFNAMES:
				cmpiutilSimpleAssocResultsForReferenceNames(handler,
														inst,
														pStatus);
				break;
			default:
				break;
		}
	}
	DEBUGOUT("Leaving cmpiutilSimpleAssocResults() called\n");
}


CMPIStatus
cmpiutilSimpleAssociators(
		doReferencesFunc doRefs,
		CMPIAssociationMI* self,
		const CMPIBroker *broker,
		const CMPIContext* context,
		const CMPIResult* results,
		const CMPIObjectPath* cop,
		const char *assocClass,
		const char *resultClass,
		const char *role,
		const char *resultRole,
		const char** properties)
{
	DEBUGOUT("In cmpiutilSimpleAssociators() called\n");
	cmpiutilSimpleAssocHandler handler;
	handler.broker = broker;
	handler.ctx = context;
	handler.properties = properties;
	handler.results = results;
	handler.origCop = cop;
	handler.role = role;
	handler.type = ASSOCS;
	DEBUGOUT("In cmpiutilSimpleAssociators()... calling doRefs()\n");
	return doRefs(&handler, self, broker, context, results, cop,
				assocClass, resultClass, role, resultRole, properties);
	DEBUGOUT("In cmpiutilSimpleAssociators() returning (back from doRefs()\n");
}

CMPIStatus
cmpiutilSimpleAssociatorNames(
		doReferencesFunc doRefs,
		CMPIAssociationMI* self,
		const CMPIBroker *broker,
		const CMPIContext* context,
		const CMPIResult* results,
		const CMPIObjectPath* cop,
		const char *assocClass,
		const char *resultClass,
		const char *role,
		const char *resultRole)
{
	cmpiutilSimpleAssocHandler handler;
	handler.broker = broker;
	handler.ctx = context;
	handler.properties = NULL;
	handler.results = results;
	handler.origCop = cop;
	handler.role = role;
	handler.type = ASSOCNAMES;
	return doRefs(&handler, self, broker, context, results, cop,
				assocClass, resultClass, role, resultRole, NULL);
}

CMPIStatus
cmpiutilSimpleReferences(
		doReferencesFunc doRefs,
		CMPIAssociationMI* self,
		const CMPIBroker *broker,
		const CMPIContext* context,
		const CMPIResult* results,
		const CMPIObjectPath* cop,
		const char *resultClass,
		const char *role,
		const char** properties)
{
	cmpiutilSimpleAssocHandler handler;
	handler.broker = broker;
	handler.ctx = context;
	handler.properties = properties;
	handler.results = results;
	handler.origCop = cop;
	handler.role = role;
	handler.type = REFS;
	// note: the incoming resultClass maps to 'assocClass' in doReferences
	return doRefs(&handler, self, broker, context, results, cop,
				resultClass, "", role, "", properties);
}

CMPIStatus
cmpiutilSimpleReferenceNames(
		doReferencesFunc doRefs,
		CMPIAssociationMI* self,
		const CMPIBroker *broker,
		const CMPIContext* context,
		const CMPIResult* results,
		const CMPIObjectPath* cop,
		const char* resultClass,
		const char* role)
{
	cmpiutilSimpleAssocHandler handler;
	handler.broker = broker;
	handler.ctx = context;
	handler.properties = NULL;
	handler.results = results;
	handler.origCop = cop;
	handler.role = role;
	handler.type = REFNAMES;
	// note: the incoming resultClass maps to 'assocClass' in doReferences
	return doRefs(&handler, self, broker, context, results, cop,
				resultClass, "", role, "", NULL);
}

